@@ -49,9 +49,8 @@ def subexon_connectivity(subexon_table, id_column='SubexonIDCluster'):
         if nrows > 1:
             for row_index in range(1, nrows):
                 connected_pairs.append(
-                    (transcript.iloc[row_index - 1,
-                                     col_index], transcript.iloc[row_index,
-                                                                 col_index]))
+                    (transcript.iloc[row_index - 1, col_index],
+                     transcript.iloc[row_index, col_index]))
     return set(connected_pairs)
 
 
@@ -686,7 +685,15 @@ def column_clusters(colpatterns):
                 cluster.end = colpattern.end
             else:
                 clusters.append(_colcluster(colpattern))
-    return _cluster_column_clusters(clusters)
+
+    n_clusters = len(clusters)
+    while True:
+        clusters = _cluster_column_clusters(clusters)
+        if n_clusters == len(clusters):
+            break
+        n_clusters = len(clusters)
+
+    return clusters
 
 
 def get_consensus(msa, threshold=0.5):
@@ -809,8 +816,8 @@ def _store_s_exons(subexon_df, seq, subexon, gene, s_exon_id):
     """
     seq = seq.replace('-', '')
     length = len(seq)
-    query = (subexon_df['SubexonIndex'] == subexon) & (subexon_df['GeneID']
-                                                       == gene)
+    query = (subexon_df['SubexonIndex'] == subexon) & (
+        subexon_df['GeneID'] == gene)
     value = subexon_df.loc[query, 'S_exons'].unique()[0]
     if '_' in value:
         subexon_df.loc[query, 'S_exons'] += '/{}'.format(s_exon_id)
