@@ -271,23 +271,21 @@ def _is_first_or_last_exon(row_list, row_index):
     end_exon = False
 
     n_rows = len(row_list)
-
     if row_index == 0:
         start_exon = True
-    elif row_index == (n_rows - 1):
+    if row_index == (n_rows - 1):
         end_exon = True
-    else:
-        row = row_list[row_index]
 
+    row = row_list[row_index]
+    if row_index > 0:
         identical_to_previous = row_list[
             row_index - 1]['TranscriptID'] == row['TranscriptID']
+        if not identical_to_previous:  # or (row['StartPhase'] == -1):
+            start_exon = True
+    if row_index < (n_rows - 1):
         identical_to_next = row_list[row_index +
                                      1]['TranscriptID'] == row['TranscriptID']
-
-        if (not identical_to_previous) or (row['StartPhase'] == -1):
-            start_exon = True
-
-        if (not identical_to_next) or (row['EndPhase'] == -1):
+        if not identical_to_next:  # or (row['EndPhase'] == -1):
             end_exon = True
 
     return start_exon, end_exon
@@ -330,9 +328,11 @@ def _manage_end_phase_negative_strand(row_list, row_index, cds_seq, end_exon):
     If the last exon of the transcript has an incomplete codon at the end, it is
     deleted on the returned sequence.
 
-    NOTE: _check_phases_by_position should be used beipt_infore this function
+    NOTE: _check_phases_by_position should be used before this function
     to ensure that there are not errors in the input data.
     """
+    if cds_seq is None:
+        return None
     end_phase = row_list[row_index]['EndPhase']
     if end_phase in {1, 2}:
         if end_exon:
@@ -368,6 +368,8 @@ def _manage_start_phase_positive_strand(row_list, row_index, cds_seq,
     >>> _manage_start_phase_positive_strand(row_list, 1, seq_two, False)
     'TGTCAAAAGACACATTCTGTAGTGTTGTAA'
     """
+    if cds_seq is None:
+        return None
     start_phase = row_list[row_index]['StartPhase']
     if start_phase in {1, 2}:
         if start_exon:
@@ -375,8 +377,11 @@ def _manage_start_phase_positive_strand(row_list, row_index, cds_seq,
             cds_seq = cds_seq[n_bases:]  # delete incomplete codon
         else:
             n_bases = phases.bases_to_complete_next_codon(start_phase)
-            previous_exon_sequence = row_list[row_index - 1]['ExonSequence']
-            cds_seq = previous_exon_sequence[-n_bases:] + cds_seq
+            if n_bases is None:
+                return None
+            seq = row_list[row_index - 1]['ExonSequence']
+            seq = seq[-n_bases:]  # pylint: disable=invalid-unary-operand-type
+            cds_seq = seq + cds_seq
 
     return cds_seq
 
@@ -398,8 +403,13 @@ def _manage_end_phase_positive_strand(cds_seq, end_phase):
     >>> _manage_end_phase_positive_strand('XXXYYY', 0)
     'XXXYYY'
     """
+    if cds_seq is None:
+        return None
     if end_phase in {1, 2}:
-        return cds_seq[:-phases.bases_to_complete_next_codon(end_phase)]
+        n_bases = phases.bases_to_complete_next_codon(end_phase)
+        if n_bases is None:
+            return None
+        return cds_seq[:-n_bases]  # pylint: disable=invalid-unary-operand-type
 
     return cds_seq  # do not change CDS sequence if exon start phase is 0 or -1
 
@@ -451,9 +461,8 @@ def add_protein_seq(data_frame,
     the output of add_exon_sequences when it is applied to the output
     of read_exon_file.
     """
-    _check_column_presence(
-        data_frame, 'ExonSequence', 'You need to run '
-        'add_exonpercent_identity_cutoff_sequences first.')
+    _check_column_presence(data_frame, 'ExonSequence', 'You need to run '
+                           'add_exon_sequences first.')
     _check_column_absence(data_frame, seq_column,
                           'Values are going to change.')
     _check_column_absence(data_frame, 'IncompleteCDS',
@@ -509,14 +518,18 @@ def add_protein_seq(data_frame,
                 cds_seq = _manage_start_phase_positive_strand(
                     row_list, row_index, cds_seq, start_exon)
 
-        # Add the translated CDS :
-        if isinstance(cds_seq, str):
-            sequences.append(Seq(cds_seq, IUPAC.extended_dna).translate())
-        else:
-            sequences.append(cds_seq.translate())
-
         # Look for signals of incomplete CDS :
-        incomplete_cds.append(_is_incomplete_cds(row, start_exon, end_exon))
+        incomplete = _is_incomplete_cds(row, start_exon, end_exon)
+        if (cds_seq is not None) and ((len(cds_seq) % 3) == 0):
+            # Add the translated CDS :
+            if isinstance(cds_seq, str):
+                sequences.append(Seq(cds_seq, IUPAC.extended_dna).translate())
+            else:
+                sequences.append(cds_seq.translate())
+        else:
+            sequences.append("")
+            incomplete = True
+        incomplete_cds.append(incomplete)
 
         row_index += 1
 
@@ -640,18 +653,18 @@ def delete_incomplete_sequences(data_frame):
         agg(lambda df: "".join([str(s) for s in df])[-1] != "*")
     # Incomplete sequences do not end with '*'
 
-    incomplete_transcripts = set(
-        incomplete_seqs.index[incomplete_seqs['ExonProteinSequence']]).union(
-            incomplete_cdss.index[incomplete_cdss['IncompleteCDS']])
-
-    if incomplete_transcripts:
-        clusters.inform_about_deletions(incomplete_transcripts,
-                                        "Incomplete transcripts were found:")
-        data_frame.drop([
-            i for i in data_frame.index
-            if data_frame.loc[i, 'TranscriptID'] in incomplete_transcripts
-        ],
-                        inplace=True)
+    if not incomplete_seqs.empty:
+        incomplete_transcripts = set(incomplete_seqs.index[
+            incomplete_seqs['ExonProteinSequence']]).union(
+                incomplete_cdss.index[incomplete_cdss['IncompleteCDS']])
+        if incomplete_transcripts:
+            clusters.inform_about_deletions(
+                incomplete_transcripts, "Incomplete transcripts were found:")
+            data_frame.drop([
+                i for i in data_frame.index
+                if data_frame.loc[i, 'TranscriptID'] in incomplete_transcripts
+            ],
+                            inplace=True)
 
 
 def delete_badquality_sequences(data_frame):
@@ -664,18 +677,18 @@ def delete_badquality_sequences(data_frame):
         groupby('TranscriptID'). \
         agg(lambda df: any('X' in str(s) for s in df))
 
-    badquality_transcripts = set(
-        badquality_seqs.index[badquality_seqs['ExonProteinSequence']])
-
-    if badquality_transcripts:
-        clusters.inform_about_deletions(
-            badquality_transcripts,
-            "Transcripts with bad quality sequences were found:")
-        data_frame.drop([
-            i for i in data_frame.index
-            if data_frame.loc[i, 'TranscriptID'] in badquality_transcripts
-        ],
-                        inplace=True)
+    if not badquality_seqs.empty:
+        badquality_transcripts = set(
+            badquality_seqs.index[badquality_seqs['ExonProteinSequence']])
+        if badquality_transcripts:
+            clusters.inform_about_deletions(
+                badquality_transcripts,
+                "Transcripts with bad quality sequences were found:")
+            data_frame.drop([
+                i for i in data_frame.index
+                if data_frame.loc[i, 'TranscriptID'] in badquality_transcripts
+            ],
+                            inplace=True)
 
 
 def find_identical_sequences(data_frame):
@@ -855,11 +868,16 @@ def read_transcript_info(  # pylint: disable=too-many-arguments
     exon_table = read_exon_file(exon_table_file)
     # Merge both tables to have all the transcript information in one
     # DataFrame and to filter the exon table based on tsl evidence:
-    tsl_table.rename(columns={'TranscriptID': 'TranscriptID'}, inplace=True)
     transcript_info = pd.merge(tsl_table,
                                exon_table,
                                how='inner',
                                on='TranscriptID')
+    # Just in case, ensure the correct order after merge:
+    transcript_info.sort_values(by=['GeneID', 'TranscriptID', 'ExonRank'],
+                                inplace=True)
+    # Delete repeated exons:
+    transcript_info.drop_duplicates(
+        subset=['GeneID', 'TranscriptID', 'ExonID'], inplace=True)
     # 3. Read sequences using BioPython and add them to the exon DataFrame:
     add_exon_sequences(transcript_info, exon_sequence_file)
     # Add protein sequence to the table:
@@ -868,11 +886,17 @@ def read_transcript_info(  # pylint: disable=too-many-arguments
     delete_incomplete_sequences(transcript_info)
     if remove_badquality:
         delete_badquality_sequences(transcript_info)
+
+    if transcript_info.empty:
+        raise ValueError("There are no transcripts left!")
+
     # Ensure exon order and check the downloaded exon phase information:
     phases.check_order_and_phases(transcript_info)
     # NOTE: delete_incomplete_sequences deletes sequences with incomplete CDSs.
     # check_order_and_phases should run after it, because it throws errors with
     # some incomplete CDSs.
+    # Delete incomplete CDS identified by check_order_and_phases:
+    delete_incomplete_sequences(transcript_info)
     # More clean up:
     delete_identical_sequences(transcript_info)
     merge_identical_exons(transcript_info)
